// Kleines Paket mit Vektorroutinen fr 3D-Berechnung
// als Klasse definiert, Implementation
// (c) 1996 by Andreas Meile, HTL Brugg-Windisch
// Datum: 18.11.1996  Klasse 3 Ia
// nderung am 14.3.1997 : ldexp() hinzugefgt

#include <stdio.h>
#include <math.h>

#include "vektor3d.h"  // Spezifikationen direkt aus dem Headerfile

// Implementation

// Konstruktor
vektor3d::vektor3d(float xk, float yk, float zk) {
  x = xk;
  y = yk;
  z = zk;
}

// Zuweisung
vektor3d & vektor3d::operator= (const vektor3d &v) {
  x = v.x;
  y = v.y;
  z = v.z;
  return *this;
}

vektor3d vektor3d::operator+ (const vektor3d &v) {
  vektor3d summe;
  summe.x = x + v.x;
  summe.y = y + v.y;
  summe.z = z + v.z;
  return summe;
}

vektor3d vektor3d::operator- (const vektor3d &v) {
  vektor3d differenz;
  differenz.x = x - v.x;
  differenz.y = y - v.y;
  differenz.z = z - v.z;
  return differenz;
}

vektor3d vektor3d::operator- () {
  vektor3d inver_vekt;
  inver_vekt.x = -x;
  inver_vekt.y = -y;
  inver_vekt.z = -z;
  return inver_vekt;
}

vektor3d vektor3d::operator* (float skalar) {
  vektor3d gestr;
  gestr.x = x * skalar;
  gestr.y = y * skalar;
  gestr.z = z * skalar;
  return gestr;
}

vektor3d operator* (float skalar, const vektor3d &v) {
  vektor3d gestr;
  gestr.x = skalar * v.x;
  gestr.y = skalar * v.y;
  gestr.z = skalar * v.z;
  return gestr;
}

// Division mit Skalar
vektor3d vektor3d::operator/ (float skalar) {
  vektor3d gesta;
  gesta.x = x / skalar;
  gesta.y = y / skalar;
  gesta.z = z / skalar;
  return gesta;
}

// Skalarprodukt
float vektor3d::operator* (const vektor3d &v) {
  return x * v.x + y * v.y + z * v.z;
}

// C-spezifische Kurzoperatoren
vektor3d & vektor3d::operator+= (const vektor3d &v) {
  x += v.x;
  y += v.y;
  z += v.z;
  return *this;
}

vektor3d & vektor3d::operator-= (const vektor3d &v) {
  x -= v.x;
  y -= v.y;
  z -= v.z;
  return *this;
}

vektor3d & vektor3d::operator*= (float skalar) {
  x *= skalar;
  y *= skalar;
  z *= skalar;
  return *this;
}

vektor3d & vektor3d::operator/= (float skalar) {
  x /= skalar;
  y /= skalar;
  z /= skalar;
  return *this;
}

vektor3d kreuzp(const vektor3d &v1, const vektor3d &v2) {
  vektor3d resul;
  resul.x = v1.y * v2.z - v1.z * v2.y;
  resul.y = v1.z * v2.x - v1.x * v2.z;
  resul.z = v1.x * v2.y - v1.y * v2.x;
  return resul;
}

// Verwendete Definition: ldexp(V, p)= V * ldexp(1.0, p)
vektor3d ldexp(const vektor3d &v, int p) {
  vektor3d resul;
  resul.x = ldexp(v.x, p);
  resul.y = ldexp(v.y, p);
  resul.z = ldexp(v.z, p);
}

// Euklidsche Norm
float vektor3d::norm() {
  return sqrt(x * x + y * y + z * z);
}

// Einheitsvektor
vektor3d vektor3d::einheit() {
  return (*this) / (this -> norm());
}

// Winkel zwischen zwei Vektoren
float winkel(vektor3d &v1, vektor3d &v2) {
  return acos((v1 * v2) / (v1.norm() * v2.norm()));
}

// Geometrische Funktionen
vektor3d vektor3d::gedreht(vektor3d &a, // Achse
                           float w) {   // Drehwinkel
  vektor3d h, h2, v2;
  h = (((*this) * a) / (a * a)) * a;
  h2 = (*this) - h;
  v2 = h + h2 * cos(w) + kreuzp(*this, a.einheit()) * sin(w);
  return v2;
}

// Ausgabe am Bildschirm
int vektor3d::put() {
  return printf("[%f,%f,%f]", x, y, z);
}
